<template>
    <div class="model-container">
        <div class="view-container" ref="threeDBox"></div>
        <div id="loading">
            <el-progress
                type="circle"
                :percentage="percent"
                :show-text="false"
                :stroke-width="8"
            ></el-progress>
        </div>
        <div
            id="tag"
            :class="[
                'tag-container',
                {'video-tag': tagData && tagData.type == 1},
            ]"
            style="display: none"
        >
            <i id="close" @click="close" class="el-icon-close"></i>
            <div class="item-info name" v-if="tagData && tagData.name">
                {{ tagData && tagData.name }}
                <span
                    v-show="tagData && tagData.type == 1"
                    :class="`status-color-${tagData.status}`"
                    >[{{ getDictName('sbzt', tagData.status) }}]</span
                >
            </div>
            <div class="item-info name" v-if="tagData && tagData.laneName">
                {{ tagData && tagData.laneName }}
            </div>
            <!-- <el-button
                :class="tagData && tagData.type == 1 ? '' : 'right'"
                type="primary"
                plain
                id="play"
                size="mini"
                @click="play(tagData)"
                >播放</el-button
            > -->
            <div class="video-wrapper" v-show="tagData && tagData.type == 1">
                <player
                    ref="player"
                    :showWdn="false"
                    :showFull="false"
                    class="player-outer"
                ></player>
            </div>
            <div
                class="item-info-wrapper"
                v-show="tagData && tagData.type !== 1"
            >
                <div class="item-info" v-if="tagData && tagData.type">
                    类型：{{ getDictName('sblx', tagData.type) }}
                </div>
                <div
                    class="item-info"
                    v-if="tagData && tagData.positionPathName"
                >
                    位置：{{ tagData && tagData.positionPathName }}
                </div>
                <div class="item-info" v-if="tagData && tagData.status">
                    状态：<span :class="`status-color-${tagData.status}`">{{
                        getDictName('sbzt', tagData.status)
                    }}</span>
                </div>

                <div
                    class="item-info"
                    v-if="tagData && (tagData.amount || tagData.amount == 0)"
                >
                    通行车辆：{{ tagData.amount }} 辆
                </div>
                <div
                    class="item-info"
                    v-if="tagData && (tagData.money || tagData.amount == 0)"
                >
                    通行费：{{ tagData.money }} 元
                </div>
            </div>
        </div>
        <div class="icon-box"></div>
        <!-- <div class="btn"><el-button @click="ploeRotate">起杆</el-button></div> -->
    </div>
</template>

<script>
import player from '@/views/Normal/components/videoPlayerDH.vue'

import * as THREE from 'three'
import {OrbitControls} from 'three/examples/jsm/controls/OrbitControls.js'
// import {FBXLoader} from 'three/examples/jsm/loaders/FBXLoader.js'
import {GLTFLoader} from 'three/examples/jsm/loaders/GLTFLoader.js'
import {
    CSS2DObject,
    CSS2DRenderer,
} from 'three/examples/jsm/renderers/CSS2DRenderer.js'
import {GUI} from 'three/examples/jsm/libs/lil-gui.module.min.js'

// 用于模型边缘高亮
import {EffectComposer} from 'three/examples/jsm/postprocessing/EffectComposer.js'
import {RenderPass} from 'three/examples/jsm/postprocessing/RenderPass.js'
import {OutlinePass} from 'three/examples/jsm/postprocessing/OutlinePass.js'
import {OutputPass} from 'three/examples/jsm/postprocessing/OutputPass.js'
import {ShaderPass} from 'three/examples/jsm/postprocessing/ShaderPass.js'
import {FXAAShader} from 'three/examples/jsm/shaders/FXAAShader.js'
import deviceTypeList from './deviceTypeList.js'
import laneData from './laneData.js'
import iconName from './iconName.js'
import utils from '@/utils.js'
import deviceAPI from '@/api/DeviceManagement/device.js'
import commonAPI from '@/api/bigScreen/common.js'
import DBUtils from './loader.js'
export default {
    name: 'ModelIndex',
    components: {player},
    data() {
        return {
            width: 0,
            height: 0,
            scene: null,
            camera: null,
            controls: null,
            renderer: null,
            css2Renderer: null,
            clock: null,
            mixer: null,
            stats: null,
            animationId: null,
            chooseObj: null,
            tag: null,
            percent: 0,
            gui: null,
            composer: null, //效果组合器
            outlinePass: null, //发光通道
            effectFXAA: null, //自定义的着色器通道
            pole1: null,
            deviceList: {}, // 设备对象
            activeList: [],
            tagData: null, //设备数据
            iconGroup: {}, // 图标对象的组合
        }
    },
    mounted() {
        this.width = window.innerWidth
        this.height = window.innerHeight
        this.init()
        this.animate()
    },
    methods: {
        getDictName(code, value) {
            return utils.getDictName(code, value)
        },
        changeDevice(list) {
            this.drop()
            this.activeList = []
            list.forEach((item) => {
                for (let key in this.deviceList[item]) {
                    let icon = this.createIcon(item)
                    this.iconGroup[this.deviceList[item][key].name] = icon
                    this.deviceList[item][key].add(
                        this.iconGroup[this.deviceList[item][key].name]
                    )
                    this.activeList.push(this.deviceList[item][key])
                }
            })
            this.pick(this.activeList)
            this.animate()
        },
        //栏杆机旋转
        ploeRotate() {
            if (this.poloState.pole1 == 1) {
                this.pole1.rotation.set(Math.PI / 2, 0, 0, 'XZY')
                this.poloState.pole1 = 0
            } else {
                this.pole1.rotation.set(0, 0, 0, 'XZY')
                this.poloState.pole1 = 1
            }
        },
        close() {
            if (this.chooseObj) {
                //把原来选中模型对应的标签和发光描边隐藏
                this.chooseObj.remove(this.tag) //从场景移除
            }
        },
        addComposer() {
            // 创建一个EffectComposer（效果组合器）对象，然后在该对象上添加后期处理通道。
            this.composer = new EffectComposer(this.renderer)
            // 新建一个场景通道  为了覆盖到原来的场景上
            let renderPass = new RenderPass(this.scene, this.camera)
            this.composer.addPass(renderPass)
            // 物体边缘发光通道
            this.outlinePass = new OutlinePass(
                new THREE.Vector2(this.width, this.height),
                this.scene,
                this.camera
            )
            this.outlinePass.visibleEdgeColor.set(parseInt(0x07e1e7)) // 呼吸显示的颜色
            this.outlinePass.hiddenEdgeColor = new THREE.Color(0, 0, 0) // 呼吸消失的颜色
            this.composer.addPass(this.outlinePass)

            // 解决高亮后环境变暗的问题
            const outputPass = new OutputPass()
            this.composer.addPass(outputPass)

            // 自定义的着色器通道 作为参数
            this.effectFXAA = new ShaderPass(FXAAShader)
            this.effectFXAA.uniforms['resolution'].value.set(
                1 / this.width,
                1 / this.height
            )
            this.composer.addPass(this.effectFXAA)
        },
        createIcon(type) {
            let name = iconName[type]
            // 创建div元素(作为标签)
            var div = document.createElement('div')
            div.style.width = '30px'
            div.style.height = '52px'
            let img = require(`@/assets/img/bigScreen/${name}.png`)
            div.style.background = `url(${img}) no-repeat center / 100% 100%`
            div.classList.add('icon-box')

            var label = new CSS2DObject(div)
            // div.style.pointerEvents = 'none'
            return label //返回CSS2模型标签
        },
        getALLDevice(gltf) {
            for (let key in deviceTypeList) {
                this.deviceList[key] = {}
                deviceTypeList[key].forEach((item) => {
                    this.deviceList[key][item] =
                        gltf.scene.getObjectByName(item)
                })
            }
        },
        loadModel() {
            const indexDB = new DBUtils('indexdb001', 'sfz-lh', 1)

            indexDB.get('./static/3DModel/sfz-lh.gltf').then((blob) => {
                const url = URL.createObjectURL(new Blob([blob]))
                const loader = new GLTFLoader()
                loader.load(
                    url,
                    (gltf) => {
                        const root = gltf.scene
                        //...(下面部分就和threejs加载glb模型代码一样，按业务需求做相应的操作)
                        this.percent = 100
                        this.getALLDevice(gltf)
                        this.createLane()
                        this.scene.add(root)
                        THREE.Cache.add('model', gltf)
                        document.getElementById('loading').style.display =
                            'none'
                        this.animate()
                    },
                    (xhr) => {
                        const {loaded, total} = xhr
                        let percent = Math.floor((loaded / total) * 100)
                        if (percent < 96) {
                            this.percent = percent
                        }
                    }
                )
            })
        },
        // loadModel() {
        //     const loader = new GLTFLoader()
        //     loader.load(
        //         './static/3DModel/sfz-lh.gltf',
        //         (gltf) => {
        //             this.percent = 100

        //             this.getALLDevice(gltf)
        //             this.createLane()

        //             // this.pole1 = gltf.scene.getObjectByName('G-__588756')

        //             this.scene.add(gltf.scene)
        //             document.getElementById('loading').style.display = 'none'
        //         },
        //         (xhr) => {
        //             let percent = Math.floor((xhr.loaded / xhr.total) * 100)
        //             if (percent < 96) {
        //                 this.percent = percent
        //             }
        //         },
        //         (error) => {
        //             console.log(error)
        //         }
        //     )
        // },
        GUIControler(mesh) {
            this.gui = new GUI()

            this.gui.domElement.style.right = '0px'
            this.gui.domElement.style.top = '100px'
            this.gui.domElement.style.width = '300px'

            this.gui.add(mesh.position, 'x', -100, 100).name('X坐标').step(0.1)
            this.gui.add(mesh.position, 'y', -100, 100).name('Y坐标').step(0.1)
            this.gui.add(mesh.position, 'z', -100, 100).name('Z坐标').step(0.1)
        },
        createLane() {
            this.deviceList['入口'] = {}
            this.deviceList['出口'] = {}
            laneData.Lanes.forEach((item) => {
                // PlaneGeometry
                const geometry = new THREE.PlaneGeometry(30, 4.5, 1, 1)
                geometry.rotateX(Math.PI / 2)
                // 材质对象Material
                const material = new THREE.MeshBasicMaterial({
                    color: 0x434446, //设置材质颜色
                    opacity: 0.01,
                    transparent: true, //开启透明
                    side: THREE.DoubleSide,
                })

                const mesh = new THREE.Mesh(geometry, material)
                mesh.name = item.name
                mesh.position.set(item.x, item.y, item.z)
                this.scene.add(mesh) //网格模型添加到场景中
                if (item.type == '入口') {
                    this.deviceList['入口'][item.name] = mesh
                } else {
                    this.deviceList['出口'][item.name] = mesh
                }
            })
        },
        createCss2Render() {
            let container = this.$refs.threeDBox
            const div = document.getElementById('tag')
            // HTML元素转化为threejs的CSS2模型对象
            this.tag = new CSS2DObject(div)
            // mesh.add(tag)
            document.getElementById('close').style.pointerEvents = 'auto'

            this.css2Renderer = new CSS2DRenderer()

            this.css2Renderer.setSize(window.innerWidth, window.innerHeight)

            container.appendChild(this.css2Renderer.domElement)

            this.css2Renderer.domElement.style.pointerEvents = 'none'
            this.css2Renderer.domElement.style.position = 'absolute'
            this.css2Renderer.domElement.style.top = '0px'
        },
        createAmbient() {
            const ambient = new THREE.AmbientLight(0xffffff, 1)
            this.scene.add(ambient)

            const directionalLight = new THREE.DirectionalLight(0xffffff, 2)
            // 设置光源的方向：通过光源position属性和目标指向对象的position属性计算
            directionalLight.position.set(80, 100, 50)
            // 方向光指向对象网格模型mesh，可以不设置，默认的位置是0,0,0
            // directionalLight.target = mesh
            this.scene.add(directionalLight)
        },
        init() {
            THREE.Cache.enabled = true //开启缓存
            let container = this.$refs.threeDBox
            this.scene = new THREE.Scene()
            // 创建纹理加载器
            const textureLoader = new THREE.TextureLoader()
            // 创建天空球
            const skyGeometry = new THREE.SphereGeometry(1000, 60, 60)
            const skyTexture = textureLoader.load(
                // './static/3DModel/img/sky.jpg'
                './static/3DModel/img/sky.png'
            ) // 天空纹理
            const skyMaterial = new THREE.MeshBasicMaterial({
                map: skyTexture, // 颜色贴图
            })
            // 反转球体，主要是反转z轴
            skyGeometry.scale(1, 1, -1)
            const sky = new THREE.Mesh(skyGeometry, skyMaterial)
            this.scene.add(sky)

            this.camera = new THREE.PerspectiveCamera(
                30,
                window.innerWidth / window.innerHeight,
                1,
                3000
            )
            // -87.0511, 4.7929, 0.799

            this.camera.position.set(-82.3681, 4.6218, 0.3505)
            this.camera.lookAt(0, 0, 0)

            this.createCss2Render() // 二维弹窗
            this.loadModel() //加载模型

            // const axesHelper = new THREE.AxesHelper(100)
            // this.scene.add(axesHelper)

            this.createAmbient()

            this.renderer = new THREE.WebGLRenderer({antialias: true})
            this.renderer.setPixelRatio(window.devicePixelRatio)
            this.renderer.setSize(window.innerWidth, window.innerHeight)

            this.renderer.outputColorSpace = THREE.SRGBColorSpace //设置为SRGB颜色空间
            container.appendChild(this.renderer.domElement)

            this.controls = new OrbitControls(
                this.camera,
                this.renderer.domElement
            )
            //-65.1083, 4.2508, 0.851
            this.controls.target = new THREE.Vector3()
            this.controls.minDistance = 0
            this.controls.maxDistance = 200
            this.controls.minPolarAngle = 0
            this.controls.maxPolarAngle = 1.55 //Math.PI
            // this.controls.enablePan = false // 移动
            this.controls.update()

            this.controls.addEventListener('change', this.animate)

            window.addEventListener('resize', this.onWindowResize)

            this.renderer.domElement.addEventListener('click', (event) => {
                // .offsetY、.offsetX以canvas画布左上角为坐标原点,单位px
                const px = event.offsetX
                const py = event.offsetY
                // 屏幕坐标px、py转WebGL标准设备坐标x、y
                // width、height表示canvas画布宽高度
                const x = (px / window.innerWidth) * 2 - 1
                const y = -(py / window.innerHeight) * 2 + 1

                //创建一个射线投射器`Raycaster`
                const raycaster = new THREE.Raycaster()

                //.setFromCamera()计算射线投射器`Raycaster`的射线属性.ray
                // 形象点说就是在点击位置创建一条射线，射线穿过的模型代表选中
                raycaster.setFromCamera(new THREE.Vector2(x, y), this.camera)
                //.intersectObjects([mesh1, mesh2, mesh3])对参数中的网格模型对象进行射线交叉计算
                // 未选中对象返回空数组[],选中一个对象，数组1个元素，选中两个对象，数组两个元素
                // const intersects = raycaster.intersectObjects([mesh])
                // const intersects = raycaster.intersectObject(this.scene, true)

                const intersects = raycaster.intersectObjects(this.activeList)
                // console.log('射线器返回的对象', intersects)
                // intersects.length大于0说明，说明选中了模型
                if (intersects.length > 0) {
                    // 选中模型的第一个模型，设置为红色
                    // this.drop()
                    // intersects[0].object.material.color.set(0xff0000)

                    this.chooseObj = intersects[0].object
                    intersects[0].object.add(this.tag)
                    this.getData()

                    // this.pick([this.chooseObj])
                } else {
                    // mesh.material.color.set(0xfff000)
                    if (this.chooseObj) {
                        //把原来选中模型对应的标签和发光描边隐藏
                        this.chooseObj.remove(this.tag) //从场景移除
                        // this.drop()
                        this.chooseObj = null
                        this.tagData = null
                    }
                }
                this.animate()
            })
            this.addComposer()
            setTimeout(() => {
                this.state = '异常'
            }, 10000)
        },
        getData() {
            let play = document.getElementById('play')
            // console.log(this.chooseObj, play)
            if (this.chooseObj.name.indexOf('lane') == -1) {
                let sysCode =
                    this.chooseObj.name.length > 10
                        ? this.chooseObj.name.slice(0, 10)
                        : this.chooseObj.name
                let queryData = {
                    pageBean: {
                        page: 1,
                        pageSize: -1,
                        showTotal: true,
                    },
                    querys: [
                        {
                            group: 'main',
                            operation: 'EQUAL',
                            parentGroup: '',
                            property: 'sysCode',
                            relation: 'AND',
                            value: sysCode,
                        },
                    ],
                }
                deviceAPI.queryDeviceList(queryData).then((res) => {
                    if (res.value && res.value.rows) {
                        this.tagData = res.value.rows[0]
                        if (this.tagData && this.tagData.type == 1) {
                            this.$nextTick(() => {
                                document.getElementById(
                                    'tag'
                                ).style.zIndex = 999
                                this.$refs.player.play(this.tagData)
                            })
                        }
                    }
                })
            } else {
                let id = this.chooseObj.name.replace('lane', '')
                let queryData = this.qs.stringify({
                    laneId: id,
                })
                commonAPI.queryLaneData(queryData).then((res) => {
                    if (res.value && res.value.length) {
                        this.tagData = res.value[0]
                    }
                })
            }
        },
        drop() {
            this.activeList.forEach((item) => {
                item.remove(this.iconGroup[item.name])
            })
            this.outlinePass.selectedObjects = []
        },
        pick(selectedObjects) {
            // 筛选出车道mesh
            let outlineObj = selectedObjects.filter(
                (item) => item.name.indexOf('lane') == -1
            )
            this.outlinePass.selectedObjects = [...outlineObj]
        },
        onWindowResize() {
            const width = window.innerWidth
            const height = window.innerHeight
            this.camera.aspect = width / height
            this.renderer.setSize(width, height)
            this.css2Renderer.setSize(width, height)
            this.composer.setSize(width, height)
            this.camera.updateProjectionMatrix()
            this.effectFXAA.uniforms['resolution'].value.set(
                1 / this.width,
                1 / this.height
            )
        },
        animate() {
            // this.animationId = requestAnimationFrame(this.animate)
            this.css2Renderer.render(this.scene, this.camera)
            this.composer.render(this.scene, this.camera)
            // this.renderer.render(this.scene, this.camera)

            // 根据相机控件可视化调试相机位置
            // console.log('controls.target', this.controls.target)
            // console.log('camera.position', this.camera.position)
        },

        showData() {
            console.log('controls.target', this.controls.target)
            console.log('camera.position', this.camera.position)
        },
        removeEventListeners() {},

        destroyThreejs() {
            try {
                this.renderer.dispose()
                this.renderer.forceContextLoss()
                this.renderer.content = null
                let gl = this.renderer.domElement.getContext('webgl')
                if (gl && gl.getExtension('WEBGL_lose_context')) {
                    gl.getExtension('WEBGL_lose_context').loseContext()
                }
                this.renderer = null
                this.camera = null
                this.scene.traverse((child) => {
                    if (child.material) {
                        child.material.dispose()
                    }
                    if (child.geometry) {
                        child.geometry.dispose()
                    }
                    child = null
                })
                this.scene = null
            } catch (e) {
                console.error('Failed to destroy threejs', e)
            }
        },
    },
    beforeDestroy() {
        cancelAnimationFrame(this.animationId)
        this.removeEventListeners()
        this.destroyThreejs()
        // this.gui.domElement.style.display = 'none'
    },
}
</script>
<style lang='scss' scoped>
.model-container{
    position: relative;
    width: 100%;
    height: 100%;
    overflow: hidden;
    .icon-box{
        position: relative;
        width: 32px;
        height: 32px;
        background: url("../../../assets/img/bigScreen/icon-warn.png") no-repeat center / 100% 100%; 
    }
    .view-container {
        width: 100%;
        height: 100%;
        overflow: hidden;
    }
    .tag-container{
        width: 348px;
        height: 168px;
        padding: 20px;
        // border: 2px solid #fff;
        top: -190px;
        z-index: 9999!important;
        
        color: #fff;
        border-radius: 10px;
        // background: url("../../../assets/img/bigScreen/tag-bg.png") no-repeat center / 100% 100%;
        background: url("../../../assets/img/bigScreen/popbg.png") no-repeat center / 100% 100%;
        // background-color: rgba(116, 130, 143, 0.9);
        overflow: hidden;
        .item-info{
            font-size: 14px;
            padding: 6px 0;
        }
        .item-info.name{
            border-bottom: 1px solid rgba(255,255,255,0.5);
            padding-top: 0px;
            padding-bottom: 10px;
        }
        .status-color-2{
            font-size: 14px;
        }
        .status-color-1{
            font-size: 14px;
            color: #07E1E7;
        }
        .status-color-3{
            font-size: 14px;
            color: #F35675;
            font-weight: bold;
        }
        .item-info-wrapper{
            border: 4px double rgba(255,255,255,0.5);// rgba(255,255,255,0.5);
            border-radius: 4px;
            margin-top: 20px;
            padding: 10px;
            box-sizing: border-box;
            // border-style: groove;
            pointer-events: none;

        }
    }
    .video-tag.tag-container{
        width: 660px;
        height: 460px;
    }
    .video-wrapper{
        width: 100%;
        height: 100%;
        pointer-events: auto;
    }
    #close{
        position: absolute;
        right: 20px;
        top: 20px;
        font-size: 16px;
        cursor: pointer;
    }
    #close:hover{
        color: #F35675;
    }
    .tag-container:after{
        content: '';
        height: 70px;
        width: 0px;
        background-color: #07E1E7;
        position: absolute;
        top: 100%;
        border: 1px solid;
        border-image: linear-gradient(180deg, rgba(7, 225, 231, 1), rgba(7, 225, 231, 0)) 2 2;
        left: calc(50% - 1px);
    }
    .tag-container:before{
        content: '';
        height: 4px;
        width: 10px;
        border-radius: 5px;
        background-color: #07E1E7;
        position: absolute;
        opacity: 1;
        filter: blur(2px);
        top: calc(100% + 70px);
        left: calc(50% - 5px);
    }
    #loading{
        position: absolute;
        left: 0;
        top: 0;
        width: 100%;
        height: 100%;
        overflow: hidden;
        color: #fff;
        background-color: rgba(40, 85, 128, 0.9);
        display: flex;
        justify-content: center;
        align-items: center;
    }
    .btn{
        position: absolute;
        left: 50%;
        bottom: 20px;
        transform: translateX(-50%);
        z-index: 999;
    }
    #play{
        cursor: pointer;
        opacity: 1;
        position: absolute;
        right: 40px;
        top: 14px;
    }
    #play.right{
        right: -80px;
    }
}

</style>